/*
  pathnames.h

  dsniff pathnames.

  Copyright (c) 2000 Dug Song <dugsong@monkey.org>

  $Id: pathnames.h,v 1.1 2000/06/14 00:29:55 dugsong Exp $
*/

#ifndef PATHNAMES_H
#define PATHNAMES_H

#ifndef DSNIFF_LIBDIR
#define DSNIFF_LIBDIR		"/usr/local/lib/"
#endif

#define DSNIFF_SERVICES		"dsniff.services"
#define DSNIFF_MAGIC		"dsniff.magic"

#endif /* PATHNAMES_H */
