/*
  asn1.h

  ASN.1 routines.
 
  Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  
  $Id: asn1.h,v 1.1 2000/05/16 17:31:13 dugsong Exp $
*/

#ifndef ASN1_H
#define ASN1_H

#define ASN1_INTEGER	2
#define ASN1_STRING	4
#define ASN1_SEQUENCE	16

int	asn1_type(u_char **buf);
int	asn1_len(u_char **buf);

#endif /* ASN1_H */

