/*
  magic.h

  Network application protocol identification, based on file(1) magic.
  
  Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  
  $Id: magic.h,v 1.1 2000/06/14 00:29:55 dugsong Exp $
*/

#ifndef MAGIC_H
#define MAGIC_H

int	 magic_init(void);

char	*magic_match(u_char *buf, int len);

#endif /* MAGIC_H */
